from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, AllParam
import tsugu_api_async

alc = Alconna(
        ["查角色"],
        Args["word#角色名，乐队，昵称等查询参数", AllParam],
        meta=CommandMeta(
            compact=True,
            description="查询角色信息",
            usage='根据关键词或角色ID查询角色信息',
            example='''查角色 10 :返回10号角色的信息。
查角色 吉他 :返回所有角色模糊搜索标签中包含吉他的角色列表。'''
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        try:
            return tsugu_api.search_character(user.displayed_server_list, text=" ".join(res.word))
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        try:
            return await tsugu_api_async.search_character(user.displayed_server_list, text=" ".join(res.word))
        except Exception as e:
            return text_response(e)

    return res





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
from tsugu_api_core._typing import _ServerName
import tsugu_api_async


alc = Alconna(
        ["查询分数表", '查分数表', '查询分数榜', '查分数榜'],
        Args["serverName;?#省略服务器名时，默认从你当前的主服务器查询。", _ServerName.__args__],
        meta=CommandMeta(
            compact=True,
            description="查询分数表",
            usage='查询指定服务器的歌曲分数表。',
            example='''查询分数表 cn :返回国服的歌曲分数表'''
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        if res.serverName:
            server = server_name_2_server_id(res.serverName)
        else:
            server = user.main_server
        try:
            return tsugu_api.song_meta(user.displayed_server_list, server)
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        if res.serverName:
            server = server_name_2_server_id(res.serverName)
        else:
            server = user.main_server
        try:
            return await tsugu_api_async.song_meta(user.displayed_server_list, server)
        except Exception as e:
            return text_response(e)

    return res





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager, AllParam
from tsugu.utils import get_user_account_list_msg
from typing import List, Tuple, Optional

alc = Alconna(
        ["解除绑定"],
        Args["any", Optional[AllParam]],
        meta=CommandMeta(
            compact=True,
            description="解除绑定",
            usage="验证解绑 记录编号(数字)",
            example="验证解绑 1 : 解绑第一个记录"
        ),
    )



def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)

        try:
            bind_record = get_user_account_list_msg(user)
            r = tsugu_api.bind_player_request(platform, user.user_id)
            return text_response(f'''请先然后登陆游戏，将 评论(个性签名) 或者 当前使用的 乐队编队名称改为
{r.get('data')['verifyCode']}
稍等片刻等待同步后，选择你要解除的账号数字：
{bind_record}
对 tsugu 发送 “验证解绑 数字” 来完成本次身份验证。
例如：验证解绑 1
''')
        except Exception as e:
            return text_response(str(e))
    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)

        try:
            bind_record = get_user_account_list_msg(user)
            r = await tsugu_api_async.bind_player_request(platform, user.user_id)
            return text_response(f'''请先然后登陆游戏，将 评论(个性签名) 或者 当前使用的 乐队编队名称改为
{r.get('data')['verifyCode']}
稍等片刻等待同步后，选择你要解除的账号数字：
{bind_record}
对 tsugu 发送 “验证解绑 数字” 来完成本次身份验证。
例如：验证解绑 1
''')
        except Exception as e:
            return text_response(str(e))
    return res




import base64
from typing import List, Union, Dict
import tarina

from .utils import *
from loguru import logger
from . import src


def handler(message: str, user_id: str, platform: str = 'red') -> List[Union[bytes, str]]:
    '''
    Tsugu Handler
    处理用户输入的自然语言，返回处理后的结果

    :param message: 用户消息
    :param user_id: 用户ID
    :param platform: 平台名称 一般为 red
    :return: List[Union[bytes, str]]
    '''
    try:
        data = handler_raw(message, user_id, platform)
        response = []
        if not data:
            return response
        for item in data:
            response.append(item['string']) if item['type'] == 'string' else None
            response.append(base64.b64decode(item['string'].encode('utf-8')) if item['type'] == 'base64' else None)
        return response
    except Exception as e:
        raise e


def handler_raw(message: str, user_id: str, platform: str = 'red') -> List[Dict[str, str]]:
    '''
    handler_raw 除了返回的数据类型不同外，其他与 handler 函数一致
    返回格式为 Tsugu 标准数据格式
    :param message: 用户消息
    :param user_id: 用户ID
    :param platform: 平台名称 一般为 red
    :return: List[Dict[str, str]]
    '''
    try:
        return r if (r := src.handler(message, user_id, platform)) else None
    except Exception as e:
        logger.error(f'Error: {e}')
        raise e


async def handler_async(message: str, user_id: str, platform: str = 'red') -> List[Union[bytes, str]]:
    '''
    Tsugu Handler Async
    异步支持
    处理用户输入的自然语言，返回处理后的结果

    :param message: 用户消息
    :param user_id: 用户ID
    :param platform: 平台名称 一般为 red
    :return: List[Union[bytes, str]]
    '''
    try:
        data = await handler_raw_async(message, user_id, platform)
        response = []
        if not data:
            return response
        for item in data:
            response.append(item['string']) if item['type'] == 'string' else None
            response.append(base64.b64decode(item['string'].encode('utf-8')) if item['type'] == 'base64' else None)
        return response
    except Exception as e:
        raise e


async def handler_raw_async(message: str, user_id: str, platform: str = 'red') -> List[Dict[str, str]]:
    '''
    handler_raw_async 除了返回的数据类型不同外，其他与 handler_async 函数一致
    返回格式为 Tsugu 标准数据格式
    :param message: 用户消息
    :param user_id: 用户ID
    :param platform: 平台名称 一般为 red
    :return: List[Dict[str, str]]
    '''
    try:
        return r if (r := await src.handler_async(message, user_id, platform)) else None
    except Exception as e:
        logger.error(f'Error: {e}')
        raise e





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
import tsugu_api_async
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager, MultiVar
from tsugu_api_core._typing import _ServerName


alc = Alconna(
        ["主服务器", "设置主服务器"],
        Args["serverName#服务器名", _ServerName],
        meta=CommandMeta(
            compact=True,
            description="主服务器",
            usage="将指定的服务器设置为你的主服务器。",
            example="""主服务器 cn : 将国服设置为主服务器。"""

        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        try:
            r = tsugu_api.change_user_data(platform, user.user_id, {'mainServer': server_name_2_server_id(res.serverName)})
        except Exception as e:
            return text_response(e)
        return text_response('主服务器已设置为 ' + res.serverName)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        try:
            r = await tsugu_api_async.change_user_data(platform, user.user_id, {'mainServer': server_name_2_server_id(res.serverName)})
        except Exception as e:
            return text_response(e)
        return text_response('主服务器已设置为 ' + res.serverName)

    return res




from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, AllParam
import tsugu_api_async


alc = Alconna(
        ["查曲"],
        Args["word#歌曲信息，名称，乐队，难度等。", AllParam],
        meta=CommandMeta(
            compact=True,
            description="查曲",
            usage='根据关键词或曲目ID查询曲目信息。',
            example='''查曲 1 :返回1号曲的信息
查曲 ag lv27 :返回所有难度为27的ag曲列表
查曲 >27 :查询大于27的曲目
查曲 滑滑蛋 :匹配歌曲 fuwa fuwa time'''
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        try:
            return tsugu_api.search_song(user.displayed_server_list, text=" ".join(res.word))
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        try:
            return await tsugu_api_async.search_song(user.displayed_server_list, text=" ".join(res.word))
        except Exception as e:
            return text_response(e)

    return res





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
import tsugu_api_async
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager, MultiVar, AllParam


alc = Alconna(
        ["绑定玩家"],
        Args["args;?", AllParam],
        meta=CommandMeta(
            compact=True,
            description="绑定玩家",
            usage="只需发送‘绑定玩家’。",
        ),
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        try:
            r = tsugu_api.bind_player_request(platform, user_id)
            return text_response(
            f'''正在绑定账号，请将 评论(个性签名) 或者 当前使用的 乐队编队名称改为\n{r.get('data')['verifyCode']}\n稍等片刻等待同步后，发送\n验证绑定 你的玩家ID(数字) 服务器名(字母缩写)\n例如：验证绑定 114****514 cn''')
        except Exception as e:
            return text_response(str(e))

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        try:
            r = await tsugu_api_async.bind_player_request(platform, user_id)
            return text_response(
            f'''正在绑定账号，请将 评论(个性签名) 或者 当前使用的 乐队编队名称改为\n{r.get('data')['verifyCode']}\n稍等片刻等待同步后，发送\n验证绑定 你的玩家ID(数字) 服务器名(字母缩写)\n例如：验证绑定 114****514 cn''')
        except Exception as e:
            return text_response(str(e))

    return res




from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, AllParam
import tsugu_api_async


alc = Alconna(
        ["查活动"],
        Args["word#请输入活动名，乐队，活动ID等查询参数", AllParam],
        meta=CommandMeta(
            compact=True,
            description="查活动",
            usage='根据关键词或活动ID查询活动信息',
            example='''查活动 绿 tsugu :返回所有属性加成为pure，且活动加成角色中包括羽泽鸫的活动列表
查活动 177 :返回177号活动的信息
查活动 >225 :查询大于225的活动
查活动 220-225 :查询220到225的活动'''
        )
    )

def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        try:
            return tsugu_api.search_event(user.displayed_server_list, text=" ".join(res.word))
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        try:
            return await tsugu_api_async.search_event(user.displayed_server_list, text=" ".join(res.word))
        except Exception as e:
            return text_response(e)

    return res





import importlib
from arclet.alconna import Arparma, output_manager, command_manager
from ..utils import text_response


# 主要功能
universal_list = [
    "get_card_illustration",
    "search_player",
    "gacha_simulate",
    "search_gacha",
    "search_event",
    "search_song",
    "song_meta",
    "search_character",
    "song_chart",
    "ycx_all",
    "ycx",
    "lsycx",
    "search_card",
    "event_stage",
]

# 远程数据库相关功能
remote_commands = [
    "player_status",
    "switch_car_forwarding_off",
    "switch_car_forwarding_on",
    "bind_player",
    "unbind_player",
    "change_server_mode",
    "change_default_server",
    "bind_player_verification_off",
    "bind_player_verification_on",
    "change_user_player_index",
    "change_user_player_list_sequence",
]

# BandoriStation 相关功能
bandoristation_commands = [
    "ycm",
    "rooms_forward",
]

# 帮助信息
help_command = [
    "help",
]


def require(module_path, cmd):
    globals()[cmd] = importlib.import_module(module_path.format(cmd=cmd), __package__)


# 导入模块
[require(f'.universal.{cmd}', cmd) for cmd in universal_list]
[require(f'.remote.{cmd}', cmd) for cmd in remote_commands]
[require(f'.bandoristation.{cmd}', cmd) for cmd in bandoristation_commands]
[require(f'.help.{cmd}', cmd) for cmd in help_command]

output_manager.set_action(lambda *_: None)


def handler(message, user_id, platform):
    union_list = universal_list + remote_commands + bandoristation_commands + help_command

    for i in union_list:
        result = getattr(globals()[i], 'handler')(message, user_id, platform)

        # 未生成结果
        if isinstance(result, Arparma):
            # 匹配了命令头
            if result.head_matched:

                # 帮助信息
                if result.error_data == ['-h']:
                    return getattr(globals()['help'], 'handler')('help ' + result.source.replace('Alconna::', '', 1), user_id, platform)
                # 错误信息
                else:
                    return text_response(str(result.error_info) + '\n' + command_manager.command_help(result.source.replace('Alconna::', '', 1)))

        # 已生成结果
        if isinstance(result, list):
            return result

    return None


async def handler_async(message, user_id, platform):
    union_list = universal_list + remote_commands + bandoristation_commands + help_command

    for i in union_list:
        result = await getattr(globals()[i], 'handler_async')(message, user_id, platform)

        # 未生成结果
        if isinstance(result, Arparma):
            # 匹配了命令头
            if result.head_matched:

                # 帮助信息
                if result.error_data == ['-h']:
                    return await getattr(globals()['help'], 'handler_async')('help ' + result.source.replace('Alconna::', '', 1), user_id, platform)
                # 错误信息
                else:
                    return text_response(str(result.error_info) + '\n' + command_manager.command_help(result.source.replace('Alconna::', '', 1)))

        # 已生成结果
        if isinstance(result, list):
            return result

    return None


import typing
from typing import List
import tsugu_api
import tsugu_api_async
from loguru import logger
import time

from tsugu_api_core._typing import _ServerId, _UserPlayerInList

_i_s = {0: "jp", 1: "en", 2: "tw", 3: "cn", 4: "kr"}
_s_i = {"jp": 0, "en": 1, "tw": 2, "cn": 3, "kr": 4}


def text_response(string):
    return [{"type": "string", "string": str(string)}]


def server_names_2_server_ids(server_name: List[str]) -> List[_ServerId]:
    return [_s_i[code] for code in server_name]


def server_name_2_server_id(server_name: str) -> _ServerId:
    return _s_i[server_name] if server_name in _s_i else None


def server_ids_2_server_names(index: List[_ServerId]) -> List[str]:
    return [_i_s[code] for code in index]


def server_id_2_server_name(index: _ServerId) -> str:
    return _i_s[index] if index in _i_s else None


def server_exists(server):
    if server or server == 0:
        return True
    if not server:
        return False
    return False


def get_user_account_list_msg(user) -> str:
    def mask_data(game_id: str):
        game_id = str(game_id)
        if len(game_id) < 6:
            return game_id[:3] + '*' * (len(game_id) - 3)
        elif len(game_id) < 3:
            return '*' * len(game_id)
        else:
            game_id = game_id[:3] + '*' * (len(game_id) - 6) + game_id[-3:]
        return game_id

    bind_record = '\n'.join(
        [f'{i + 1}. {mask_data(str(x.get("playerId")))} {server_id_2_server_name(x.get("server"))}' for i, x in
         enumerate(user.user_player_list)])

    return bind_record

class User:
    def __init__(self, user_id: str, platform: str, main_server: _ServerId, displayed_server_list: List[_ServerId],
                 share_room_number: bool, user_player_index: int, user_player_list: List[_UserPlayerInList]):
        self.user_id = user_id
        self.platform = platform
        self.main_server = main_server
        self.displayed_server_list = displayed_server_list
        self.share_room_number = share_room_number
        self.user_player_index = user_player_index
        self.user_player_list = user_player_list


def get_user(user_id: str, platform: str) -> User:
    '''
    获取用户数据
    多次尝试获取用户数据
    兼容旧版用户数据

    :param user_id:
    :param platform:
    :return:
    '''
    for i in range(3):
        try:
            user_data_res = tsugu_api.get_user_data(platform, user_id)
            user_data = user_data_res.get('data')
            user = User(user_id=user_id,
                        platform=platform,
                        main_server=user_data.get('mainServer'),
                        displayed_server_list=user_data.get('displayedServerList'),
                        share_room_number=user_data.get('shareRoomNumber'),
                        user_player_index=user_data.get('userPlayerIndex'),
                        user_player_list=user_data.get('userPlayerList'))
            return user
        except TimeoutError:
            time.sleep(0.2)
            continue
        except Exception as e:
            logger.error(f'Error: {e}')
            raise e


async def get_user_async(user_id: str, platform: str) -> User:
    '''
    获取用户数据
    多次尝试获取用户数据
    兼容旧版用户数据
W
    :param user_id:
    :param platform:
    :return:
    '''
    for i in range(3):
        try:
            user_data_res = await tsugu_api_async.get_user_data(platform, user_id)
            user_data = user_data_res.get('data')
            user = User(user_id=user_id,
                        platform=platform,
                        main_server=user_data.get('mainServer'),
                        displayed_server_list=user_data.get('displayedServerList'),
                        share_room_number=user_data.get('shareRoomNumber'),
                        user_player_index=user_data.get('userPlayerIndex'),
                        user_player_list=user_data.get('userPlayerList'))
            return user
        except TimeoutError:
            time.sleep(0.2)
            continue
        except Exception as e:
            logger.error(f'Error: {e}')
            raise e





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager



alc = Alconna(
        ["查试炼", '查stage', '查舞台', '查festival', '查5v5'],
        Args["eventId;?#省略活动ID时查询当前活动。", [int]]["meta;?#歌曲meta。", ['-m']],

        meta=CommandMeta(
            compact=True,
            description="查试炼",
            usage='查询当前服务器当前活动试炼信息。',
            example='''查试炼 157 -m :返回157号活动的试炼信息，包含歌曲meta。
查试炼 -m :返回当前活动的试炼信息，包含歌曲meta。
查试炼 :返回当前活动的试炼信息。'''
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        if res.meta:
            meta = True
        else:
            meta = False
        try:
            return tsugu_api.event_stage(user.main_server, res.eventId, meta)
        except Exception as e:
            return text_response(e)

    return res

async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        if res.meta:
            meta = True
        else:
            meta = False
        try:
            return await tsugu_api_async.event_stage(user.main_server, res.eventId, meta)
        except Exception as e:
            return text_response(e)

    return res




from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, server_exists, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
from tsugu_api_core._typing import _ServerName


alc = Alconna(
        ["玩家状态"],
        Args["accountIndex;?", int],
        meta=CommandMeta(
            compact=True,
            description="查询自己的玩家状态",
            usage='根据关键词或活动ID查询活动信息',
            example='''玩家状态 :返回指定默认账号的玩家状态
玩家状态 1 :返回账号1的玩家状态

可以使用 交换绑定顺序 来交换绑定的玩家顺序
例如：交换绑定顺序 1 2 交换账号1和2的绑定顺序

可以使用 主账号 来设置默认查询账号
例如：主账号 2 设置账号2为默认查询账号

已取消自动跟随服务器显示不同服务器账号“玩家状态”的功能，请使用 “玩家状态 数字” 来查询。
例如：玩家状态 2
'''
        )
    )


def handler(message: str, user_id: str, platform: str):

    def _case_default():
        user_player_index = user.user_player_index
        try:
            game_id_msg = user.user_player_list[user_player_index]
            return tsugu_api.search_player(int(game_id_msg.get("playerId")), game_id_msg.get("server")) + text_response(f'已查找默认玩家状态（{user_player_index + 1}），“help 玩家状态” 了解更多。')
        except Exception as e:
            return text_response(e)

    def _case_index():
        if res.accountIndex > len(user.user_player_list) or res.accountIndex < 1:
            return text_response(f'未找到记录 {res.accountIndex}。')
        try:
            game_id_msg = user.user_player_list[res.accountIndex - 1]
            return tsugu_api.search_player(int(game_id_msg.get("playerId")), game_id_msg.get("server")) + text_response(f'已查找账号 {res.accountIndex} 玩家状态，“help 玩家状态” 了解更多。')
        except Exception as e:
            return text_response(e)

    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        if res.accountIndex:
            return _case_index()
        else:
            return _case_default()

    return res


async def handler_async(message: str, user_id: str, platform: str):

        async def _case_default():
            user_player_index = user.user_player_index
            try:
                game_id_msg = user.user_player_list[user_player_index]
                return await tsugu_api_async.search_player(int(game_id_msg.get("playerId")), game_id_msg.get("server")) + text_response(f'已查找默认玩家状态（{user_player_index + 1}），“help 玩家状态” 了解更多。')
            except Exception as e:
                return text_response(e)

        async def _case_index():
            if res.accountIndex > len(user.user_player_list) or res.accountIndex < 1:
                return text_response(f'未找到记录 {res.accountIndex}。')
            try:
                game_id_msg = user.user_player_list[res.accountIndex - 1]
                return await tsugu_api_async.search_player(int(game_id_msg.get("playerId")), game_id_msg.get("server")) + text_response(f'已查找账号 {res.accountIndex} 玩家状态，“help 玩家状态” 了解更多。')
            except Exception as e:
                return text_response(e)

        res = alc.parse(message)

        if res.matched:
            user = await get_user_async(user_id, platform)
            if res.accountIndex:
                return _case_index()
            else:
                return _case_default()

        return res





from typing import List

from ...utils import get_user, text_response, User, server_id_2_server_name, server_names_2_server_ids, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager, \
    MultiVar
from tsugu_api_core._typing import _ServerName, _PartialTsuguUser, _ChangeUserDataResponse
from tsugu.utils import get_user_account_list_msg

alc = Alconna(
    ["交换绑定顺序", "交换账号顺序"],
    Args["accountList;?#使用空格两个索引。", List[int]],
    meta=CommandMeta(
        compact=True,
        description="设定信息显示中的账号排序",
        example="""交换绑定顺序 1 2 : 将第一个账号和第二个账号交换顺序。"""

    )
)


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        if not res.accountList or len(res.accountList) != 2:
            user = get_user(user_id, platform)
            account_list_msg = get_user_account_list_msg(user)
            return text_response(f'''请提供两个账号的索引。
{account_list_msg}''')

        user = get_user(user_id, platform)
        index_1, index_2 = res.accountList
        if any([index < 1 for index in res.accountList]):
            return text_response('账号索引不正确。')

        user.user_player_list[index_1 - 1], user.user_player_list[index_2 - 1] = user.user_player_list[index_2 - 1], \
        user.user_player_list[index_1 - 1]
        update = {'userPlayerList': user.user_player_list}
        try:
            tsugu_api.change_user_data(platform, user.user_id, update)
            return text_response('账号顺序已交换！')
        except Exception as e:
            return text_response(str(e))

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        if not res.accountList or len(res.accountList) != 2:
            user = await get_user_async(user_id, platform)
            account_list_msg = get_user_account_list_msg(user)
            return text_response(f'''请提供两个账号的索引。
{account_list_msg}''')

        user = await get_user_async(user_id, platform)
        index_1, index_2 = res.accountList
        if any([index < 1 for index in res.accountList]):
            return text_response('账号索引不正确。')

        user.user_player_list[index_1 - 1], user.user_player_list[index_2 - 1] = user.user_player_list[index_2 - 1], \
        user.user_player_list[index_1 - 1]
        update = {'userPlayerList': user.user_player_list}
        try:
            await tsugu_api_async.change_user_data(platform, user.user_id, update)
            return text_response('账号顺序已交换！')
        except Exception as e:
            return text_response(str(e))




from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager, MultiVar, AllParam


alc = Alconna(
        ["help"],
        Args["cmd;?#命令", str],
        meta=CommandMeta(
            compact=True,
            description="Chat Tsugu Py 帮助",)
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        if not res.cmd:
            return text_response(command_manager.all_command_help())

        try:
            cmd = command_manager.command_help(res.cmd)
            return text_response(cmd)

        except KeyError:
            # return text_response('未找到该命令')
            return None

        except Exception as e:
            # return text_response('未知错误')
            return None

    return None


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        if not res.cmd:
            return text_response(command_manager.all_command_help())

        try:
            cmd = command_manager.command_help(res.cmd)
            return text_response(cmd)

        except KeyError:
            # return text_response('未找到该命令')
            return None

        except Exception as e:
            # return text_response('未知错误')
            return None

    return None



from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
import tsugu_api_async


alc = Alconna(
        ["查卡池"],
        Args["gachaId#可以通过查活动、查卡等获取", str],
        meta=CommandMeta(
            compact=True,
            description="查卡池",
            usage='根据卡池ID查询卡池信息',
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        try:
            return tsugu_api.search_gacha(user.displayed_server_list, res.gachaId)
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        try:
            return await tsugu_api_async.search_gacha(user.displayed_server_list, res.gachaId)
        except Exception as e:
            return text_response(e)

    return res





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager, AllParam
import tsugu_api_async


alc = Alconna(
        ["查卡", "查卡牌"],
        Args["word#请输入卡面ID，角色等查询参数，使用空格隔开", AllParam],
        meta=CommandMeta(
            compact=True,
            description="查询卡片信息。",
            usage='根据关键词或卡牌ID查询卡片信息。',
            example='''查卡 1399 :返回1399号卡牌的信息。
    查卡面 1399 :返回1399号卡牌的插画。
    查卡 红 ars 5x :返回角色 ars 的 5x 卡片的信息。'''
        )
    )

def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        try:
            return tsugu_api.search_card(user.displayed_server_list, " ".join(res.word))
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        try:
            return await tsugu_api_async.search_card(user.displayed_server_list, " ".join(res.word))
        except Exception as e:
            return text_response(e)

    return res




from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
from tsugu_api_core._typing import _ServerName
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
import tsugu_api_async


alc = Alconna(
        ["查玩家", "查询玩家"],
        Args["playerId#你的游戏账号(数字)", int]["serverName;?#省略服务器名时，默认从你当前的主服务器查询。", _ServerName.__args__],
        meta=CommandMeta(
            compact=True,
            description="查询玩家信息",
            usage='查询指定ID玩家的信息。查询指定ID玩家的信息。',
            example='查玩家 40474621 jp : 查询日服玩家ID为40474621的玩家信息。\n查玩家 10000000 : 查询你当前默认服务器中，玩家ID为10000000的玩家信息。',
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        if res.serverName:
            server = server_name_2_server_id(res.serverName)
        else:
            server = user.main_server
        print(res.playerId, server)
        if str(res.playerId).startswith('4') and server == 3:
            return text_response('Bestdori 暂不支持渠道服相关功能。')
        try:
            return tsugu_api.search_player(res.playerId, server)
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        if res.serverName:
            server = server_name_2_server_id(res.serverName)
        else:
            server = user.main_server
        if str(res.playerId).startswith('4') and server == 3:
            return text_response('Bestdori 暂不支持渠道服相关功能。')
        try:
            return await tsugu_api_async.search_player(res.playerId, server)
        except Exception as e:
            return text_response(e)

    return res






from ...utils import text_response, User, server_id_2_server_name, server_name_2_server_id
import tsugu_api
import tsugu_api_async
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
from tsugu.utils import get_user, get_user_async

alc = Alconna(
        ["ycm", "车来", "有车吗"],
        meta=CommandMeta(
            compact=True,
            description="获取车牌",
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        try:
            data = tsugu_api.query_room_number()
        except Exception as e:
            return text_response('获取房间信息失败，请稍后再试')
        if not data:
            return text_response('myc')

        new_data = {}
        user = get_user(user_id, platform)
        for i in data:
            if len(user.user_player_list) >= user.user_player_index + 1:  # 只添加有玩家数据的信息
                new_data.update({'player': {'playerId': user.user_player_list[user.user_player_index]['playerId'],
                                            'server': user.user_player_list[user.user_player_index]['server']}})
            new_data.update({'number': int(i['number'])})
            new_data.update({'source': i['source_info']['name']})
            new_data.update({'userId': i['user_info']['user_id']})
            new_data.update({'time': i['time']})
            new_data.update({'avatarUrl': 'https://asset.bandoristation.com/images/user-avatar/' + i['user_info']['avatar']})
            new_data.update({'userName': i['user_info']['username']})
            new_data.update({'rawMessage': i['raw_message']})
        # 过滤掉 number 相同的
        try:
            return tsugu_api.room_list([new_data])
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        try:
            data = await tsugu_api_async.query_room_number()
        except Exception as e:
            return text_response('获取房间信息失败，请稍后再试')
        if not data:
            return text_response('myc')

        new_data = {}
        user = await get_user_async(user_id, platform)
        for i in data:
            if len(user.user_player_list) >= user.user_player_index + 1:  # 只添加有玩家数据的信息
                new_data.update({'player': {'playerId': user.user_player_list[user.user_player_index]['playerId'], 'server': user.user_player_list[user.user_player_index]['server']}})
            new_data.update({'number': int(i['number'])})
            new_data.update({'source': i['source_info']['name']})
            new_data.update({'userId': i['user_info']['user_id']})
            new_data.update({'time': i['time']})
            new_data.update({'avatarUrl': 'https://asset.bandoristation.com/images/user-avatar/' + i['user_info']['avatar']})
            new_data.update({'userName': i['user_info']['username']})
            new_data.update({'rawMessage': i['raw_message']})
        # 过滤掉 number 相同的
        try:
            return await tsugu_api_async.room_list([new_data])
        except Exception as e:
            return text_response(e)

    return res


from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager


alc = Alconna(
        ["关闭车牌转发", "关闭个人车牌转发"],
        meta=CommandMeta(
            compact=True,
            description="关闭车牌转发",)
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        update = {'shareRoomNumber': False, }
        try:
            r = tsugu_api.change_user_data(platform, user.user_id, update)
            return text_response('关闭车牌转发成功！')
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        update = {'shareRoomNumber': False, }
        try:
            r = await tsugu_api_async.change_user_data(platform, user.user_id, update)
            return text_response('关闭车牌转发成功！')
        except Exception as e:
            return text_response(e)

    return res





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager


alc = Alconna(
        ["验证解绑"],
        Args["index#要解绑的绑定编号", int],
        meta=CommandMeta(
            compact=True,
            description="验证解绑",
            usage="验证解绑 记录编号(数字)",
            example="验证解绑 1 : 解绑第一个记录"
        ),
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        if len(user.user_player_list) < int(res.index):
            return text_response('未找到记录')
        player_id = user.user_player_list[int(res.index) - 1].get("playerId")
        server = user.user_player_list[int(res.index) - 1].get("server")
        # print(platform, user.user_id, player_id, server)
        # print(type(platform), type(user.user_id), type(player_id), type(server))
        try:
            r = tsugu_api.bind_player_verification(platform, user.user_id, server, player_id, 'unbind')
            return text_response('解绑成功！')
        except Exception as e:
            return text_response(e)
    
    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        if len(user.user_player_list) < int(res.index):
            return text_response('未找到记录')
        player_id = user.user_player_list[int(res.index) - 1].get("playerId")
        server_mode = user.user_player_list[int(res.index) - 1].get("server")
        try:
            r = await tsugu_api_async.bind_player_verification(platform, user.user_id, server_mode, player_id, 'unbind')
            return text_response('解绑成功！')
        except Exception as e:
            return text_response(str(e))

    return res




from typing import List

from ...utils import get_user, text_response, User, server_id_2_server_name, server_names_2_server_ids, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager, MultiVar
from tsugu_api_core._typing import _ServerName, _PartialTsuguUser, _ChangeUserDataResponse
from tsugu.utils import get_user_account_list_msg


alc = Alconna(
    ["主账号"],
    Args["accountIndex#主账号，从1开始", int],
    meta=CommandMeta(
        compact=True,
        description="设定默认玩家状态、车牌展示中的主账号使用第几个账号",
        example="""主账号 2 : 将第二个账号设置为主账号。"""
    )
)


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        if len(user.user_player_list) < res.accountIndex or res.accountIndex < 1:
            return text_response('账号数量不正确。')

        update = {'userPlayerIndex': res.accountIndex - 1}
        try:
            tsugu_api.change_user_data(platform, user.user_id, update)
            return text_response('主账号已设置为账号 ' + str(res.accountIndex))
        except Exception as e:
            return text_response(str(e))

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        if len(user.user_player_list) < res.accountIndex or res.accountIndex < 1:
            return text_response('账号数量不正确。')

        update = {'userPlayerIndex': res.accountIndex - 1}
        try:
            await tsugu_api_async.change_user_data(platform, user.user_id, update)
            return text_response('主账号已设置为账号 ' + str(res.accountIndex))
        except Exception as e:
            return text_response(str(e))

    return res



from ...utils import get_user, text_response, User, server_id_2_server_name, server_names_2_server_ids, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager, MultiVar
from tsugu_api_core._typing import _ServerName, _PartialTsuguUser, _ChangeUserDataResponse


alc = Alconna(
        ["设置默认服务器", "默认服务器"],
        Args["serverList#使用空格分隔服务器列表。", MultiVar(_ServerName)],
        meta=CommandMeta(
            compact=True,
            description="设定信息显示中的默认服务器排序",
            example="""设置默认服务器 cn jp : 将国服设置为第一服务器，日服设置为第二服务器。"""

        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        try:
            tsugu_api.change_user_data(platform, user.user_id, {'displayedServerList': server_names_2_server_ids(res.serverList)})  # serverList是res的用户输入
        except Exception as e:
            return text_response(str(e))

        return text_response('默认服务器已设置为 ' + ' '.join(res.serverList))
    
    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        try:
            await tsugu_api_async.change_user_data(platform, user.user_id, {'displayedServerList': server_names_2_server_ids(res.serverList)})
        except Exception as e:
            return text_response(str(e))

        return text_response('默认服务器已设置为 ' + ' '.join(res.serverList))

    return res





from ...utils import text_response, User, get_user, get_user_async
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
import re
import tsugu_api
import tsugu_api_async


# 虚空注册 ##################### 虚空注册 只为了help
alc = Alconna(
    ["上传车牌"],
    meta=CommandMeta(
        description="上传车牌",
       ),
    )


def handler(message: str, user_id: str, platform: str):
    if message.startswith("上传车牌"):
        message = message[4:].strip()

    # 检查car_config['car']中的关键字
    for keyword in _car_config["car"]:
        if str(keyword) in message:
            break
    else:
        return None

    # 检查car_config['fake']中的关键字
    for keyword in _car_config["fake"]:
        if str(keyword) in message:
            return None

    pattern = r"^\d{5}(\D|$)|^\d{6}(\D|$)"
    if not re.match(pattern, message):
        return None

    user = get_user(user_id, platform)

    # 获取用户数据
    try:
        if platform:
            if not user.share_room_number:
                return None
    except Exception as e:
        # 默认不开启关闭车牌，继续提交
        pass

    try:
        car_id = message[:6]
        if not car_id.isdigit() and car_id[:5].isdigit():
            car_id = car_id[:5]
        if user.user_id.isdigit():
            car_user_id = user.user_id
        else:
            car_user_id = '3889000770'
        tsugu_api.submit_room_number(number=int(car_id), user_id=car_user_id, raw_message=message, source="Tsugu", token="ZtV4EX2K9Onb")
        return None

    except Exception as e:
        return None


async def handler_async(message: str, user_id: str, platform: str):
    if message.startswith("上传车牌"):
        message = message[4:].strip()

    # 检查car_config['car']中的关键字
    for keyword in _car_config["car"]:
        if str(keyword) in message:
            break
    else:
        return None

    # 检查car_config['fake']中的关键字
    for keyword in _car_config["fake"]:
        if str(keyword) in message:
            return None

    pattern = r"^\d{5}(\D|$)|^\d{6}(\D|$)"
    if not re.match(pattern, message):
        return None

    user = await get_user_async(user_id, platform)

    # 获取用户数据
    try:
        if platform:
            if not user.share_room_number:
                return None
    except Exception as e:
        # 默认不开启关闭车牌，继续提交
        pass

    try:
        car_id = message[:6]
        if not car_id.isdigit() and car_id[:5].isdigit():
            car_id = car_id[:5]
        if user.user_id.isdigit():
            car_user_id = user.user_id
        else:
            car_user_id = '3889000770'
        await tsugu_api_async.submit_room_number(number=int(car_id), user_id=car_user_id, raw_message=message, source="Tsugu", token="ZtV4EX2K9Onb")
        return None

    except Exception as e:
        return None


_car_config = {
    "car": [
        "q1",
        "q2",
        "q3",
        "q4",
        "Q1",
        "Q2",
        "Q3",
        "Q4",
        "缺1",
        "缺2",
        "缺3",
        "缺4",
        "差1",
        "差2",
        "差3",
        "差4",
        "3火",
        "三火",
        "3把",
        "三把",
        "打满",
        "清火",
        "奇迹",
        "中途",
        "大e",
        "大分e",
        "exi",
        "大分跳",
        "大跳",
        "大a",
        "大s",
        "大分a",
        "大分s",
        "长途",
        "生日车",
        "军训",
        "禁fc",
    ],
    "fake": [
        "114514",
        "野兽",
        "恶臭",
        "1919",
        "下北泽",
        "粪",
        "糞",
        "臭",
        "11451",
        "xiabeize",
        "雀魂",
        "麻将",
        "打牌",
        "maj",
        "麻",
        "[",
        "]",
        "断幺",
        "qq.com",
        "腾讯会议",
        "master",
        "疯狂星期四",
        "离开了我们",
        "日元",
        "av",
        "bv",
    ],
}





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
import tsugu_api_async


alc = Alconna(
        ["查卡面", "查插画"],
        Args["cardId", int],
        meta=CommandMeta(
            compact=True,
            description="查卡面",
            usage='根据卡片ID查询卡片插画',
            example='查卡面 1399 :返回1399号卡牌的插画'
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        try:
            return tsugu_api.get_card_illustration(res.cardId)
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        try:
            return await tsugu_api_async.get_card_illustration(res.cardId)
        except Exception as e:
            return text_response(e)

    return res





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
from tsugu_api_core._typing import _ServerName


alc = Alconna(
        ["验证绑定"],
        Args["playerId#你的玩家ID(数字)", int]["serverName#服务器名(字母缩写)", _ServerName],
        meta=CommandMeta(
            compact=True,
            description="验证绑定",)
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        server = server_name_2_server_id(res.serverName)
        if str(res.playerId).startswith('4') and server == 3:
            return text_response('Bestdori 暂不支持渠道服相关功能。')
        try:
            r = tsugu_api.bind_player_verification(platform, user.user_id, server, res.playerId, 'bind')
            return text_response(f'绑定成功！现在可以使用 玩家状态 {len(user.user_player_list) + 1} 查看绑定的玩家状态')
        except Exception as e:
            return text_response(str(e))

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        server = server_name_2_server_id(res.serverName)
        if str(res.playerId).startswith('4') and len(str(res.playerId)) == 10 and server == 3:
            return text_response('Bestdori 暂不支持渠道服相关功能。')
        try:
            r = await tsugu_api_async.bind_player_verification(platform, user.user_id, server, res.playerId, 'bind')
            return text_response(f'绑定成功！现在可以使用 玩家状态 {len(user.user_player_list) + 1} 查看绑定的玩家状态')
        except Exception as e:
            return text_response(str(e))

    return res






from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
from tsugu_api_core._typing import _ServerName
import tsugu_api_async


alc = Alconna(
        ["lsycx", "历史预测线"], ['/', ''],
        Args['tier', int]['eventId;?#活动ID，省略时查询当前活动。', int]
            ['serverName;?#省略服务器名时，默认从你当前的主服务器查询。', _ServerName.__args__],
        meta=CommandMeta(
            compact=True,
            description="查询指定档位的历史预测线。",
            usage='查询指定档位的预测线与最近的4期活动类型相同的活动的档线数据。',
            example='''lsycx 1000 
lsycx 1000 177 jp'''
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        server = server_name_2_server_id(res.serverName) if res.serverName else user.main_server
        try:
            return tsugu_api.cutoff_list_of_recent_event(server, res.tier, res.eventId)
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        server = server_name_2_server_id(res.serverName) if res.serverName else user.main_server
        try:
            return await tsugu_api_async.cutoff_list_of_recent_event(server, res.tier, res.eventId)
        except Exception as e:
            return text_response(e)


    return res





from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
import tsugu_api_async

from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager


alc = Alconna(
        ["抽卡模拟", "卡池模拟"],
        Args["times", int, 10]['gacha_id;?#如果没有卡池ID的话，卡池为当前活动的卡池。', int],
        meta=CommandMeta(
            compact=True,
            description="抽卡模拟",
            usage='模拟抽卡',
            example='抽卡模拟 300 922 :模拟抽卡300次，卡池为922号卡池。'
        )
    )


def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        if res.gacha_id:
            gacha_id = res.gacha_id
        else:
            gacha_id = None
        try:
            return tsugu_api.gacha_simulate(user.main_server, res.times, gacha_id)
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        if res.gacha_id:
            gacha_id = res.gacha_id
        else:
            gacha_id = None
        try:
            return await tsugu_api_async.gacha_simulate(user.main_server, res.times, gacha_id)
        except Exception as e:
            return text_response(e)

    return res




from ...utils import get_user, text_response, User, server_id_2_server_name, server_name_2_server_id, get_user_async
import tsugu_api
from arclet.alconna import Alconna, Option, Subcommand, Args, CommandMeta, Empty, Namespace, namespace, command_manager
import tsugu_api_async


difficulty_text_tuple = ('easy', 'ez', 'normal', 'nm', 'hard', 'hd', 'expert', 'ex', 'special', 'sp')
difficulty_text_2_difficulty_id = {
    "easy": 0,
    "ez": 0,
    "normal": 1,
    "nm": 1,
    "hard": 2,
    "hd": 2,
    "expert": 3,
    "ex": 3,
    "special": 4,
    "sp": 4,
}


alc = Alconna(
        ["查谱面", "查铺面"],
        Args["songId", int]['difficultyText', difficulty_text_tuple, 'ex'],
        meta=CommandMeta(
            compact=True,
            description="查谱面",
            usage='根据曲目ID与难度查询铺面信息。',
            example='''查谱面 1 :返回1号曲的ex难度谱面
查谱面 128 special :返回128号曲的special难度谱面'''
        )
    )
def handler(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = get_user(user_id, platform)
        try:
            return tsugu_api.song_chart(user.displayed_server_list, res.songId, difficulty_text_2_difficulty_id[res.difficultyText])
        except Exception as e:
            return text_response(e)

    return res


async def handler_async(message: str, user_id: str, platform: str):
    res = alc.parse(message)

    if res.matched:
        user = await get_user_async(user_id, platform)
        try:
            return await tsugu_api_async.song_chart(user.displayed_server_list, res.songId, difficulty_text_2_difficulty_id[res.difficultyText])
        except Exception as e:
            return text_response(e)

    return res





